"""Traditional-based search space.
"""

import copy

import opytimizer.utils.logging as l
from opytimizer.core import Space

logger = l.get_logger(__name__)


class SearchSpace(Space):
    """A SearchSpace class for agents, variables and methods
    related to the search space.

    """

    def __init__(self, n_agents, n_variables, lower_bound, upper_bound):
        """Initialization method.

        Args:
            n_agents (int): Number of agents.
            n_variables (int): Number of decision variables.
            lower_bound (float, list, tuple, np.array): Minimum possible values.
            upper_bound (float, list, tuple, np.array): Maximum possible values.

        """

        logger.info('Overriding class: Space -> SearchSpace.')

        # Defines missing override arguments
        n_dimensions = 1

        # Override its parent class with the receiving arguments
        super(SearchSpace, self).__init__(n_agents, n_variables, n_dimensions,
                                          lower_bound, upper_bound)

        # Builds the class
        self.build()

        logger.info('Class overrided.')

    def _initialize_agents(self):
        """Initializes agents with their positions and defines a best agent.

        """

        # Iterates through all agents
        for agent in self.agents:
            # Initializes the agent
            agent.fill_with_uniform()

        # Defines a best agent
        self.best_agent = copy.deepcopy(self.agents[0])
